searchData={"content_type":"text/markdown","items":[{"doc":"RedactEx provides protocols and derivation utilities to work with\nsensitive data that should be redacted in logs and external facing tools\n\n* `RedactEx.Redactor` module gives you automagical generation of redacting\n   functions under a desired module namespace\n* `RedactEx.Redactable` is the protocol for which implementation and\n   derivation should be created to ensure the best possible practices\n   in your codebase","ref":"RedactEx.html","title":"RedactEx","type":"module"},{"doc":"RedactEx usual protection consists in two or more steps:","ref":"RedactEx.html#module-protect-your-data","title":"Protect your data - RedactEx","type":"module"},{"doc":"You can generate functions to redact your specific data using `RedactEx.Redactor` module.\n\n    iex> defmodule MyApp.Redacting do\n    ...>    @moduledoc false\n    ...>    use RedactEx.Redactor, redactors: [\n    ...>      {\"redact_three\", length: 3, algorithm: :simple},\n    ...>      {\"redact\", lengths: 1..3, algorithm: :simple}\n    ...>    ]\n    ...> end\n\nThis will generate optimized functions and/or functions adopting the standards\nand algorithms defined in this library.\nIn particular:\n\n * `redact_three/1` will be created with an optimized match for strings of length 3 and a non optimized fallback function\n   for strings with generic length\n * `redact/1` will be created with three optimized match for strings of lengths 1, 2 and 3 and a non optimized fallback function\n   for strings with generic length","ref":"RedactEx.html#module-generate-your-redactor-functions","title":"Generate your redactor functions - RedactEx","type":"module"},{"doc":"The recommended way of protecting your structs is\n\n#","ref":"RedactEx.html#module-protect-your-structs","title":"Protect your structs - RedactEx","type":"module"},{"doc":"You can use the `@derive` macro from the `RedactEx.Redactable` protocol,\nconfigured based on your needs and optionally using functions from a module generated with\n`RedactEx.Redactor` helpers, or manually define an implementation of choice, e.g.\n\n    iex> defmodule MyRedactorModule do\n    ...>   def redact_function_one(_), do: \"(redacted1)\"\n    ...>   def redact_function_two(_), do: \"(redacted2)\"\n    ...> end\n    ...> defmodule MyApp.RedactStruct do\n    ...>   @derive {RedactEx.Redactable,\n    ...>     fields: [\n    ...>       myfield1: {MyRedactorModule, :redact_function_one},\n    ...>       myfield2: {MyRedactorModule, :redact_function_two},\n    ...>     ]}\n    ...>   defstruct [:myfield1, :myfield2]\n    ...> end\n\nor\n\n    iex> defmodule MyApp.OtherRedactStruct do\n    ...>   defstruct [:myfield1, :myfield2]\n    ...> end\n    ...> defimpl RedactEx.Redactable, for: MyApp.OtherRedactStruct do\n    ...>   def redact(_value), do: %MyApp.OtherRedactStruct{myfield1: \"(redacted)\", myfield2: \"(redacted)\"}\n    ...> end\n\n#","ref":"RedactEx.html#module-1-derive-or-implement-redactex-redactable","title":"1. Derive or implement `RedactEx.Redactable` - RedactEx","type":"module"},{"doc":"No strategy can eliminate the risk of leaking sensitive data with code only.\n\nOur suggested approach is to derive also the [inspect](https://hexdocs.pm/elixir/1.14.0/Inspect.html) protocol\nand use it for your struct whenever you log or send data to external systems, e.g.\n\n    iex> defimpl Inspect,\n    ...>   for: MyApp.OtherRedactStruct do\n    ...>   alias MyApp.OtherRedactStruct\n    ...>   alias RedactEx.Redactable\n    ...>\n    ...>   @spec inspect(OtherRedactStruct.t(), Inspect.Opts.t()) :: Inspect.Algebra.t()\n    ...>   def inspect(input, opts) do\n    ...>     input\n    ...>     |> Redactable.redact()\n    ...>     |> Inspect.Any.inspect(opts)\n    ...>   end\n    ...> end\n\nAnother strategy could be to directly call `RedactEx.Redactable.redact/1` on structs when logging, but using\n`inspect` seems to us more natural and idiomatic.\n\nAnother strategy could be to `redact`/`inspect` your data directly in a custom Logger backend, or whatever the system\nused for exporting potentially sensitive data.","ref":"RedactEx.html#module-2-use-inspect-protocol-at-your-advantage","title":"2. Use `inspect` protocol at your advantage - RedactEx","type":"module"},{"doc":"Dispatcher of supported algorithms","ref":"RedactEx.Algorithms.html","title":"RedactEx.Algorithms","type":"module"},{"doc":"","ref":"RedactEx.Algorithms.html#algorithm_simple/0","title":"RedactEx.Algorithms.algorithm_simple/0","type":"function"},{"doc":"","ref":"RedactEx.Algorithms.html#base_algorithms/0","title":"RedactEx.Algorithms.base_algorithms/0","type":"function"},{"doc":"Get the AST of a redacting function from given module and configuration","ref":"RedactEx.Algorithms.html#generate_ast/2","title":"RedactEx.Algorithms.generate_ast/2","type":"function"},{"doc":"Validate an algorithm at configuration level","ref":"RedactEx.Algorithms.html#valid_algorithm?/1","title":"RedactEx.Algorithms.valid_algorithm?/1","type":"function"},{"doc":"Behaviour to implement for an algorithm to be configured as a redacting algorithm\nwhen using RedactEx","ref":"RedactEx.Algorithms.Algorithm.html","title":"RedactEx.Algorithms.Algorithm","type":"behaviour"},{"doc":"Returns the AST of the implementing algorithm, given whatever\nsupported configuration","ref":"RedactEx.Algorithms.Algorithm.html#c:generate_ast/1","title":"RedactEx.Algorithms.Algorithm.generate_ast/1","type":"callback"},{"doc":"Check that this behaviour is implemented in given module","ref":"RedactEx.Algorithms.Algorithm.html#implemented_by?/1","title":"RedactEx.Algorithms.Algorithm.implemented_by?/1","type":"function"},{"doc":"Parses additional configuration at compile time for given algorithm.\nConfiguration will be kept in Context's `extra` field\nShall return any configuration (eventually) readable by `generate_ast` and `generate_fallback_ast`\n\nThe function shall raise in case of errors","ref":"RedactEx.Algorithms.Algorithm.html#c:parse_extra_configuration!/1","title":"RedactEx.Algorithms.Algorithm.parse_extra_configuration!/1","type":"callback"},{"doc":"","ref":"RedactEx.Algorithms.Algorithm.html#t:t/0","title":"RedactEx.Algorithms.Algorithm.t/0","type":"type"},{"doc":"Simple redacting algorithm that will keep the initial and last parts of a string and\nsubstitutes the remaining part with given character\n\n```\n# e.g.\nvalue_to_redact -> va***********ct\n```","ref":"RedactEx.Algorithms.Center.html","title":"RedactEx.Algorithms.Center","type":"module"},{"doc":"Simple redacting algorithm that will keep the initial part of a string and\nsubstitutes the remaining part with given character\n\n```\n# e.g.\nvalue_to_redact -> valu************\n```","ref":"RedactEx.Algorithms.Simple.html","title":"RedactEx.Algorithms.Simple","type":"module"},{"doc":"Configuration helpers for handling RedactEx configuration","ref":"RedactEx.Configuration.html","title":"RedactEx.Configuration","type":"module"},{"doc":"Return configurations grouped by name, so we can generate matching and catchall in a good order\nwhen needed","ref":"RedactEx.Configuration.html#parse/3","title":"RedactEx.Configuration.parse/3","type":"function"},{"doc":"Context for generating a redacting function","ref":"RedactEx.Configuration.Context.html","title":"RedactEx.Configuration.Context","type":"module"},{"doc":"For fixed-length redacted strings, returns the length of the plaintext\nto be kept of the original information","ref":"RedactEx.Configuration.Context.html#get_plaintext_length_redacted_length/3","title":"RedactEx.Configuration.Context.get_plaintext_length_redacted_length/3","type":"function"},{"doc":"Gets the actual redacted part of the string given configuration","ref":"RedactEx.Configuration.Context.html#get_redactor_string/2","title":"RedactEx.Configuration.Context.get_redactor_string/2","type":"function"},{"doc":"","ref":"RedactEx.Configuration.Context.html#t:t/0","title":"RedactEx.Configuration.Context.t/0","type":"type"},{"doc":"# Redactable\n\nProtocol for defining a redact-able item, e.g. an item whose internal elements could be redacted\nIt shall return a redact-ed item of the same type as the input item\n\nYou can derive Redactable protocol for a struct by using\n\n``` elixir\ndefmodule MyApp.RedactStruct do\n  @derive {RedactEx.Redactable,\n    fields: [\n      myfield1: {MyModule, :redact_function_one},\n      myfield2: {MyModule, :redact_function_two},\n  ]}\n  defstruct [:myfield1, :myfield2]\nend\n```\n\noptions\n\n* `:fields`    possible values of `fields` are a keyword list of configurations, where configuration can be\n  * `{module, function}`, e.g.\n\n    ``` elixir\n    fields: [\n        myfield1: {MyModule, :redact_function_one},\n        myfield2: {MyModule, :redact_function_two},\n    ]\n    ```\n\n    In this case redacting will be applied to `myfield1` and `myfield2`  using module.function as configured\n  * `:redact`, e.g.\n\n    ``` elixir\n    fields: [\n      field: :redact\n    ]\n    ```\n\n    In this case it is expected for value in `field` to have an Impl for `RedactEx.Redactable`, and `RedactEx.Redactable.redact/1` will be called\n  * `:drop`, e.g.\n\n    ``` elixir\n    fields: [\n      field: :drop\n    ]\n    ```\n\n    In this case `:field` key is entirely dropped. **NOTE** that this also means\n    struct information will be lost, and the value will become a map\n\ne.g. suppose you define a module like\n\n```\n# ./test/support/derive/redact_struct.ex\ndefmodule MyApp.RedactStruct do\n  @moduledoc false\n\n  # For usage in RedactEx.Redactable doctests\n  @derive {RedactEx.Redactable,\n          fields: [\n            myfield: {String, :reverse}\n          ]}\n  defstruct [:myfield]\nend\n```\n\nthen you can expect it to have derived the `RedactEx.Redactable` protocol\n\n    ```\n    iex> %MyApp.RedactStruct{myfield: \"reverseme\"} |> RedactEx.Redactable.redact() |> Map.get(:myfield)\n    \"emesrever\"\n    ```","ref":"RedactEx.Redactable.html","title":"RedactEx.Redactable","type":"protocol"},{"doc":"Good practices could be to implement the Inspect protocol for sensitive data and use the derived `redact`\ncapabilities of the inspected module inside its implementation.\nYou can both `derive` single fields in your struct, or define whatever implementation fits best\nfor your structs as `defimpl RedactEx.Redactable, for: MyStruct`\n\nThen you can enforce `inspect`ing structures in your logs or export functions. Unluckily this is not a simple practice\nto enforce in an automatic flavour.","ref":"RedactEx.Redactable.html#module-best-practices","title":"Best Practices - RedactEx.Redactable","type":"protocol"},{"doc":"Redacts `value` hiding sensitive information","ref":"RedactEx.Redactable.html#redact/1","title":"RedactEx.Redactable.redact/1","type":"function"},{"doc":"","ref":"RedactEx.Redactable.html#t:any_container/0","title":"RedactEx.Redactable.any_container/0","type":"type"},{"doc":"","ref":"RedactEx.Redactable.html#t:binary_container/0","title":"RedactEx.Redactable.binary_container/0","type":"type"},{"doc":"","ref":"RedactEx.Redactable.html#t:container/0","title":"RedactEx.Redactable.container/0","type":"type"},{"doc":"","ref":"RedactEx.Redactable.html#t:t/0","title":"RedactEx.Redactable.t/0","type":"type"},{"doc":"# RedactEx\n\nUtility that defines a set of functions to fast-redact data where possible, e.g. when you already know\nthe length of the string you're gonna have as validated input\nRedacting is unidirectional and is meant to help the safety of logs and exported data\n\nNote that right now this function matches on binary data, so you probably want to consider it if\nUTF-stuff is heavily involved when defining lengths.\nFor example, in extreme cases, considering that a UTF-8 string may have a string length of 5 (e.g. `héllo`) but a byte size\nof 6 (as by [documentation](https://hexdocs.pm/elixir/1.14.0/String.html#module-utf-8-encoded-and-encodings))\n\n    iex> string = \"héllo\"\n    \"héllo\"\n    iex> String.length(string)\n    5\n    iex> byte_size(string)\n    7\n\nSo you may consider it when evaluating for which kind of data you're planning to use byte-based helpers","ref":"RedactEx.Redactor.html","title":"RedactEx.Redactor","type":"module"},{"doc":"#","ref":"RedactEx.Redactor.html#module-algorithms","title":"Algorithms - RedactEx.Redactor","type":"module"},{"doc":"Those algorithms will match on the byte_size of the string\n\n| Algorithm | Description |\n| --------- | ----------- |\n| `:simple` | keeps the first `keep`% of the string and sets the rest to `redactor`, according to `redacted_size` |\n| `:center` | splits `keep`% of the string between head and tail, and sets the middle to `redactor`, according to `redacted_size` |","ref":"RedactEx.Redactor.html#module-optimized-algorithms-for-strings-with-expected-length-s","title":"Optimized algorithms for strings with expected length(s) - RedactEx.Redactor","type":"module"},{"doc":"Options:\n `redactors`  a list of redacting rules. Each element can be:\n              - a tuple in the form {redactor_function_name, configuration}\n              - a tuple in the form {[redactor_function_names], configuration}\n              - a single redactor_function_name\n              `redactor_function_name` can be an atom or string value in all cases\n\n#","ref":"RedactEx.Redactor.html#module-configuration","title":"Configuration - RedactEx.Redactor","type":"module"},{"doc":"Configuration of a redactor is a keyword list containing\n\n  * `redactor`        string                              character used to redact the hidden part of the. Defaults to `*`\n  * `redacted_size`   integer | :auto                     length of the resulting string that will be set as redacted. Defaults to `auto`,\n                                                          which will set it to `expected_string_length - keep`\n  * `algorithm`       atom | module                       algorithm used to redact the string. Defaults to `simple`\n                                                          if a module is given, it must implement the `RedactEx.Algorithms.Algorithm` behaviour\n                                                          Supported atoms are\n                                                          - `:simple` (alias for `RedactEx.Algorithms.Simple`)\n                                                          - `:center` (alias for `RedactEx.Algorithms.Center`)\n  * `keep`            integer                             quote of the string that will be kept. Defaults to `25%`\n  * `length`          integer or  `:*`                    length of the string to be considered. `:*` stands for the fallback function configuration. Is overridden by key `:lengths` if present\n  * `lengths`         [integer or `:*`] | min..max        lengths of the strings to be considered. `:*` stands for the fallback function configuration. A function for each specific length will be generated\n  * `except`          list(atom())                        list of environments for which this configuration will have effect\n  * `fallback_value`  string                              fixed value for redacting non-string values\n\n#","ref":"RedactEx.Redactor.html#module-common-configuration","title":"Common configuration - RedactEx.Redactor","type":"module"},{"doc":"iex> defmodule MyApp.Redacting do\n    ...>    @moduledoc false\n    ...>    use RedactEx.Redactor, redactors: [\n    ...>      {\"redact_three\", length: 3, algorithm: :simple},\n    ...>      {\"redact\", lengths: 1..3, algorithm: :simple}\n    ...>    ]\n    ...> end\n    iex> MyApp.Redacting.redact(\"test\")\n    ...> \"t***\"\n\n    This will expose the following functions:\n         * `MyApp.Redacting.redact` with fast matches on input of length 1, 2, 3\n         * `MyApp.Redacting.redact_three` with fast matches on input of length 3\n         * both `redact/1` and `redact_three/1` will have a slower fallback function for input of different length","ref":"RedactEx.Redactor.html#module-example","title":"Example - RedactEx.Redactor","type":"module"},{"doc":"You can define custom algorithms to define your own implementations. Take a look at `RedactEx.Algorithms.Algorithm` behaviour.","ref":"RedactEx.Redactor.html#module-custom-algorithms","title":"Custom algorithms - RedactEx.Redactor","type":"module"},{"doc":"[![CI status](https://drone-1.prima.it/api/badges/primait/redact_ex/status.svg?branch=master)](https://drone-1.prima.it/primait/redact_ex) [![Hex Docs](https://img.shields.io/badge/hex.pm-docs-brightgreen)](https://hexdocs.pm/redact_ex)\n\n# redact_ex\n\nUtilities to redact sensitive data in Elixir projects","ref":"readme.html","title":"README","type":"extras"},{"doc":"By *redacting* we consider here the practice of hiding part of a potentially sensitive information, while keeping a small part\nof it to allow understanding that it may be a valid one.\n\nFor example, you may want to *redact* a telephone number in your logs, but keep the initial digits (e.g. the prefix) because that information is relevant\nto  your understanding of the system, and at the same moment it does not expose any sensitive information.","ref":"readme.html#redacting","title":"Redacting - README","type":"extras"},{"doc":"","ref":"readme.html#configuration","title":"Configuration - README","type":"extras"},{"doc":"Full documentation is available on [hexdocs](https://hexdocs.pm/redact_ex)\n\nSee [RedactEx](./lib/redact_ex.ex) for general information about this library and its intended use\nSee [RedactEx.Redactor](./lib/redact_ex/redactor.ex) for generating fast redactors for strings\nSee [RedactEx.Redactable](./lib/redact_ex/redactable.ex) protocol for deriving redactable structs","ref":"readme.html#usage","title":"Usage - README","type":"extras"},{"doc":"The recommended way to work is to use [asdf](https://github.com/asdf-vm/asdf) language runtime versions manager.\n\nYou will find a [.tool-versions.recommended](./tool-versions.recommended) file that - hopefully - is up to date with the versions used in CI.\n\nPlease be sure that your test pass with those versions before pushing to save yourself the wait for automatic checks.\n\n``` bash\n# Use project's defaults\ncp .tool-versions.recommended .tool-versions\n```","ref":"readme.html#contributing","title":"Contributing - README","type":"extras"},{"doc":"- [ ] Expand explanation of fallback custom implementations of algorithms\n- [ ] fix dialyzer :(","ref":"readme.html#todo","title":"TODO - README","type":"extras"},{"doc":"* Other ways of redacting (e.g. first X letters and last Y exposed, random redactors, ecc)\n   * maybe-email redactor\n   * define a default strict implementation for `redact` in the `Any` implementation?\n   * some other default magic configuration in derive, e.g. a default module+function?\n   * better handling of fallbacks for derived structs, and macro-generated helpers to \"try_redact\" structs that may be nil or other","ref":"readme.html#open-points","title":"Open Points - README","type":"extras"}]}